Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Linq
Imports System.Windows
Imports System.Windows.Resources
Imports System.Xml
Imports System.Xml.Linq
Imports DemoApp.Model

Namespace DemoApp.DataAccess
	''' <summary>
	''' Represents a source of customers in the application.
	''' </summary>
	Public Class CustomerRepository
		#Region "Fields"

		Private ReadOnly _customers As List(Of Customer)

		#End Region ' Fields

		#Region "Constructor"

		''' <summary>
		''' Creates a new repository of customers.
		''' </summary>
		''' <param name="customerDataFile">The relative path to an XML resource file that contains customer data.</param>
		Public Sub New(ByVal customerDataFile As String)
			_customers = LoadCustomers(customerDataFile)
		End Sub

		#End Region ' Constructor

		#Region "Public Interface"

		''' <summary>
		''' Raised when a customer is placed into the repository.
		''' </summary>
		Public Event CustomerAdded As EventHandler(Of CustomerAddedEventArgs)

		''' <summary>
		''' Places the specified customer into the repository.
		''' If the customer is already in the repository, an
		''' exception is not thrown.
		''' </summary>
		Public Sub AddCustomer(ByVal customer As Customer)
			If customer Is Nothing Then
				Throw New ArgumentNullException("customer")
			End If

			If (Not _customers.Contains(customer)) Then
				_customers.Add(customer)

				If Me.CustomerAddedEvent IsNot Nothing Then
					RaiseEvent CustomerAdded(Me, New CustomerAddedEventArgs(customer))
				End If
			End If
		End Sub

		''' <summary>
		''' Returns true if the specified customer exists in the
		''' repository, or false if it is not.
		''' </summary>
		Public Function ContainsCustomer(ByVal customer As Customer) As Boolean
			If customer Is Nothing Then
				Throw New ArgumentNullException("customer")
			End If

			Return _customers.Contains(customer)
		End Function

		''' <summary>
		''' Returns a shallow-copied list of all customers in the repository.
		''' </summary>
		Public Function GetCustomers() As List(Of Customer)
			Return New List(Of Customer)(_customers)
		End Function

		#End Region ' Public Interface

		#Region "Private Helpers"

		Private Shared Function LoadCustomers(ByVal customerDataFile As String) As List(Of Customer)
			' In a real application, the data would come from an external source,
			' but for this demo let's keep things simple and use a resource file.
			Using stream As Stream = GetResourceStream(customerDataFile)
			Using xmlRdr As XmlReader = New XmlTextReader(stream)
				Return ( _
						From customerElem In XDocument.Load(xmlRdr).Element("customers").Elements("customer") _
						Select Customer.CreateCustomer(CDbl(customerElem.Attribute("totalSales")), CStr(customerElem.Attribute("firstName")), CStr(customerElem.Attribute("lastName")), CBool(customerElem.Attribute("isCompany")), CStr(customerElem.Attribute("email")))).ToList()
			End Using
			End Using
		End Function

		Private Shared Function GetResourceStream(ByVal resourceFile As String) As Stream
			Dim uri As New Uri(resourceFile, UriKind.RelativeOrAbsolute)

			Dim info As StreamResourceInfo = Application.GetResourceStream(uri)
			If info Is Nothing OrElse info.Stream Is Nothing Then
				Throw New ApplicationException("Missing resource file: " & resourceFile)
			End If

			Return info.Stream
		End Function

		#End Region ' Private Helpers
	End Class
End Namespace