Imports DemoApp.Properties
Imports System
Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Text.RegularExpressions

Namespace DemoApp.Model
    ''' <summary>
    ''' Represents a customer of a company.  This class
    ''' has built-in validation logic. It is wrapped
    ''' by the CustomerViewModel class, which enables it to
    ''' be easily displayed and edited by a WPF user interface.
    ''' </summary>
    Public Class Customer
        Implements IDataErrorInfo
#Region "Creation"

        Public Shared Function CreateNewCustomer() As Customer
            Return New Customer()
        End Function

        Public Shared Function CreateCustomer(ByVal totalSales As Double, ByVal firstName As String, ByVal lastName As String, ByVal isCompany As Boolean, ByVal email As String) As Customer
            Return New Customer() With _
                { _
                    .TotalSales = totalSales, _
                    .FirstName = firstName, _
                    .LastName = lastName, _
                    .IsCompany = isCompany, _
                    .Email = email _
                }

        End Function

        Protected Sub New()
        End Sub

#End Region ' Creation

#Region "State Properties"

        ''' <summary>
        ''' Gets/sets the e-mail address for the customer.
        ''' </summary>
        Private privateEmail As String
        Public Property Email() As String
            Get
                Return privateEmail
            End Get
            Set(ByVal value As String)
                privateEmail = value
            End Set
        End Property

        ''' <summary>
        ''' Gets/sets the customer's first name.  If this customer is a 
        ''' company, this value stores the company's name.
        ''' </summary>
        Private privateFirstName As String
        Public Property FirstName() As String
            Get
                Return privateFirstName
            End Get
            Set(ByVal value As String)
                privateFirstName = value
            End Set
        End Property

        ''' <summary>
        ''' Gets/sets whether the customer is a company or a person.
        ''' The default value is false.
        ''' </summary>
        Private privateIsCompany As Boolean
        Public Property IsCompany() As Boolean
            Get
                Return privateIsCompany
            End Get
            Set(ByVal value As Boolean)
                privateIsCompany = value
            End Set
        End Property

        ''' <summary>
        ''' Gets/sets the customer's last name.  If this customer is a 
        ''' company, this value is not set.
        ''' </summary>
        Private privateLastName As String
        Public Property LastName() As String
            Get
                Return privateLastName
            End Get
            Set(ByVal value As String)
                privateLastName = value
            End Set
        End Property

        ''' <summary>
        ''' Returns the total amount of money spent by the customer.
        ''' </summary>
        Private privateTotalSales As Double
        Public Property TotalSales() As Double
            Get
                Return privateTotalSales
            End Get
            Private Set(ByVal value As Double)
                privateTotalSales = value
            End Set
        End Property

#End Region ' State Properties

#Region "IDataErrorInfo Members"

        Private ReadOnly Property IDataErrorInfo_Error() As String Implements IDataErrorInfo.Error
            Get
                Return Nothing
            End Get
        End Property

        Public ReadOnly Property IDataErrorInfo_Item(ByVal propertyName As String) As String Implements IDataErrorInfo.Item
            Get
                Return Me.GetValidationError(propertyName)
            End Get
        End Property

#End Region ' IDataErrorInfo Members

#Region "Validation"

        ''' <summary>
        ''' Returns true if this object has no validation errors.
        ''' </summary>
        Public ReadOnly Property IsValid() As Boolean
            Get
                For Each [property] As String In ValidatedProperties
                    If GetValidationError([property]) IsNot Nothing Then
                        Return False
                    End If
                Next [property]

                Return True
            End Get
        End Property

        Private Shared ReadOnly ValidatedProperties() As String = {"Email", "FirstName", "LastName"}

        Private Function GetValidationError(ByVal propertyName As String) As String
            If Array.IndexOf(ValidatedProperties, propertyName) < 0 Then
                Return Nothing
            End If

            Dim [error] As String = Nothing

            Select Case propertyName
                Case "Email"
                    [error] = Me.ValidateEmail()

                Case "FirstName"
                    [error] = Me.ValidateFirstName()

                Case "LastName"
                    [error] = Me.ValidateLastName()

                Case Else
                    Debug.Fail("Unexpected property being validated on Customer: " & propertyName)
            End Select

            Return [error]
        End Function

        Private Function ValidateEmail() As String
            If IsStringMissing(Me.Email) Then
                Return Strings.Customer_Error_MissingEmail
            ElseIf (Not IsValidEmailAddress(Me.Email)) Then
                Return Strings.Customer_Error_InvalidEmail
            End If
            Return Nothing
        End Function

        Private Function ValidateFirstName() As String
            If IsStringMissing(Me.FirstName) Then
                Return Strings.Customer_Error_MissingFirstName
            End If
            Return Nothing
        End Function

        Private Function ValidateLastName() As String
            If Me.IsCompany Then
                If (Not IsStringMissing(Me.LastName)) Then
                    Return Strings.Customer_Error_CompanyHasNoLastName
                End If
            Else
                If IsStringMissing(Me.LastName) Then
                    Return Strings.Customer_Error_MissingLastName
                End If
            End If
            Return Nothing
        End Function

        Private Shared Function IsStringMissing(ByVal value As String) As Boolean
            Return String.IsNullOrEmpty(value) OrElse value.Trim() = String.Empty
        End Function

        Private Shared Function IsValidEmailAddress(ByVal email As String) As Boolean
            If IsStringMissing(email) Then
                Return False
            End If

            ' This regex pattern came from: http://haacked.com/archive/2007/08/21/i-knew-how-to-validate-an-email-address-until-i.aspx
            Dim pattern As String = "^(?!\.)(""([^""\r\\]|\\[""\r\\])*""|([-a-z0-9!#$%&'*+/=?^_`{|}~]|(?<!\.)\.)*)(?<!\.)@[a-z0-9][\w\.-]*[a-z0-9]\.[a-z][a-z\.]*[a-z]$"

            Return Regex.IsMatch(email, pattern, RegexOptions.IgnoreCase)
        End Function

#End Region ' Validation
    End Class
End Namespace