Imports System.Linq
Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports DemoApp.DataAccess
Imports DemoApp.Properties

Namespace DemoApp.ViewModel
	''' <summary>
	''' Represents a container of CustomerViewModel objects
	''' that has support for staying synchronized with the
	''' CustomerRepository.  This class also provides information
	''' related to multiple selected customers.
	''' </summary>
	Public Class AllCustomersViewModel
		Inherits WorkspaceViewModel
		#Region "Fields"

		Private ReadOnly _customerRepository As CustomerRepository

		#End Region ' Fields

		#Region "Constructor"

		Public Sub New(ByVal customerRepository As CustomerRepository)
			If customerRepository Is Nothing Then
				Throw New ArgumentNullException("customerRepository")
			End If

			MyBase.DisplayName = Strings.AllCustomersViewModel_DisplayName

			_customerRepository = customerRepository

			' Subscribe for notifications of when a new customer is saved.
			AddHandler _customerRepository.CustomerAdded, AddressOf OnCustomerAddedToRepository

			' Populate the AllCustomers collection with CustomerViewModels.
			Me.CreateAllCustomers()
		End Sub

		Private Sub CreateAllCustomers()
			Dim all As List(Of CustomerViewModel) = ( _
					From cust In _customerRepository.GetCustomers() _
					Select New CustomerViewModel(cust, _customerRepository)).ToList()

			For Each cvm As CustomerViewModel In all
				AddHandler cvm.PropertyChanged, AddressOf OnCustomerViewModelPropertyChanged
			Next cvm

			Me.AllCustomers = New ObservableCollection(Of CustomerViewModel)(all)
			AddHandler Me.AllCustomers.CollectionChanged, AddressOf OnCollectionChanged
		End Sub

		#End Region ' Constructor

		#Region "Public Interface"

		''' <summary>
		''' Returns a collection of all the CustomerViewModel objects.
		''' </summary>
		Private privateAllCustomers As ObservableCollection(Of CustomerViewModel)
		Public Property AllCustomers() As ObservableCollection(Of CustomerViewModel)
			Get
				Return privateAllCustomers
			End Get
			Private Set(ByVal value As ObservableCollection(Of CustomerViewModel))
				privateAllCustomers = value
			End Set
		End Property

		''' <summary>
		''' Returns the total sales sum of all selected customers.
		''' </summary>
		Public ReadOnly Property TotalSelectedSales() As Double
			Get
				Return Me.AllCustomers.Sum(Function(custVM)If(custVM.IsSelected, custVM.TotalSales, 0.0))
			End Get
		End Property

		#End Region ' Public Interface

		#Region " Base Class Overrides"

		Protected Overrides Sub OnDispose()
			For Each custVM As CustomerViewModel In Me.AllCustomers
				custVM.Dispose()
			Next custVM

			Me.AllCustomers.Clear()
			RemoveHandler Me.AllCustomers.CollectionChanged, AddressOf OnCollectionChanged

			RemoveHandler _customerRepository.CustomerAdded, AddressOf OnCustomerAddedToRepository
		End Sub

		#End Region ' Base Class Overrides

		#Region "Event Handling Methods"

		Private Sub OnCollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
			If e.NewItems IsNot Nothing AndAlso e.NewItems.Count <> 0 Then
				For Each custVM As CustomerViewModel In e.NewItems
					AddHandler custVM.PropertyChanged, AddressOf OnCustomerViewModelPropertyChanged
				Next custVM
			End If

			If e.OldItems IsNot Nothing AndAlso e.OldItems.Count <> 0 Then
				For Each custVM As CustomerViewModel In e.OldItems
					RemoveHandler custVM.PropertyChanged, AddressOf OnCustomerViewModelPropertyChanged
				Next custVM
			End If
		End Sub

		Private Sub OnCustomerViewModelPropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
			Dim IsSelected As String = "IsSelected"

			' Make sure that the property name we're referencing is valid.
			' This is a debugging technique, and does not execute in a Release build.
			TryCast(sender, CustomerViewModel).VerifyPropertyName(IsSelected)

			' When a customer is selected or unselected, we must let the
			' world know that the TotalSelectedSales property has changed,
			' so that it will be queried again for a new value.
			If e.PropertyName = IsSelected Then
				Me.OnPropertyChanged("TotalSelectedSales")
			End If
		End Sub

		Private Sub OnCustomerAddedToRepository(ByVal sender As Object, ByVal e As CustomerAddedEventArgs)
			Dim viewModel = New CustomerViewModel(e.NewCustomer, _customerRepository)
			Me.AllCustomers.Add(viewModel)
		End Sub

		#End Region ' Event Handling Methods
	End Class
End Namespace