
Imports System
Imports System.ComponentModel
Imports System.Windows.Input
Imports DemoApp.DataAccess
Imports DemoApp.Model
Imports DemoApp.Properties

Namespace DemoApp.ViewModel
	''' <summary>
	''' A UI-friendly wrapper for a Customer object.
	''' </summary>
	Public Class CustomerViewModel
		Inherits WorkspaceViewModel
		Implements IDataErrorInfo
		#Region "Fields"

		Private ReadOnly _customer As Customer
		Private ReadOnly _customerRepository As CustomerRepository
		Private _customerType As String
		Private _customerTypeOptions() As String
		Private _isSelected As Boolean
		Private _saveCommand As RelayCommand

		#End Region ' Fields

		#Region "Constructor"

		Public Sub New(ByVal customer As Customer, ByVal customerRepository As CustomerRepository)
			If customer Is Nothing Then
				Throw New ArgumentNullException("customer")
			End If

			If customerRepository Is Nothing Then
				Throw New ArgumentNullException("customerRepository")
			End If

			_customer = customer
			_customerRepository = customerRepository
			_customerType = Strings.CustomerViewModel_CustomerTypeOption_NotSpecified
		End Sub

		#End Region ' Constructor

		#Region "Customer Properties"

		Public Property Email() As String
			Get
				Return _customer.Email
			End Get
			Set(ByVal value As String)
				If value = _customer.Email Then
					Return
				End If

				_customer.Email = value

				MyBase.OnPropertyChanged("Email")
			End Set
		End Property

		Public Property FirstName() As String
			Get
				Return _customer.FirstName
			End Get
			Set(ByVal value As String)
				If value = _customer.FirstName Then
					Return
				End If

				_customer.FirstName = value

				MyBase.OnPropertyChanged("FirstName")
			End Set
		End Property

		Public ReadOnly Property IsCompany() As Boolean
			Get
				Return _customer.IsCompany
			End Get
		End Property

		Public Property LastName() As String
			Get
				Return _customer.LastName
			End Get
			Set(ByVal value As String)
				If value = _customer.LastName Then
					Return
				End If

				_customer.LastName = value

				MyBase.OnPropertyChanged("LastName")
			End Set
		End Property

		Public ReadOnly Property TotalSales() As Double
			Get
				Return _customer.TotalSales
			End Get
		End Property

		#End Region ' Customer Properties

		#Region "Presentation Properties"

		''' <summary>
		''' Gets/sets a value that indicates what type of customer this is.
		''' This property maps to the IsCompany property of the Customer class,
		''' but also has support for an 'unselected' state.
		''' </summary>
		Public Property CustomerType() As String
			Get
				Return _customerType
			End Get
			Set(ByVal value As String)
				If value = _customerType OrElse String.IsNullOrEmpty(value) Then
					Return
				End If

				_customerType = value

				If _customerType = Strings.CustomerViewModel_CustomerTypeOption_Company Then
					_customer.IsCompany = True
				ElseIf _customerType = Strings.CustomerViewModel_CustomerTypeOption_Person Then
					_customer.IsCompany = False
				End If

				MyBase.OnPropertyChanged("CustomerType")

				' Since this ViewModel object has knowledge of how to translate
				' a customer type (i.e. text) to a Customer object's IsCompany property,
				' it also must raise a property change notification when it changes
				' the value of IsCompany.  The LastName property is validated 
				' differently based on whether the customer is a company or not,
				' so the validation for the LastName property must execute now.
				MyBase.OnPropertyChanged("LastName")
			End Set
		End Property

		''' <summary>
		''' Returns a list of strings used to populate the Customer Type selector.
		''' </summary>
		Public ReadOnly Property CustomerTypeOptions() As String()
			Get
				If _customerTypeOptions Is Nothing Then
					_customerTypeOptions = New String() { Strings.CustomerViewModel_CustomerTypeOption_NotSpecified, Strings.CustomerViewModel_CustomerTypeOption_Person, Strings.CustomerViewModel_CustomerTypeOption_Company }
				End If
				Return _customerTypeOptions
			End Get
		End Property

        Public Overrides Property DisplayName() As String
            Get
                If Me.IsNewCustomer Then
                    Return Strings.CustomerViewModel_DisplayName
                ElseIf _customer.IsCompany Then
                    Return _customer.FirstName
                Else
                    Return String.Format("{0}, {1}", _customer.LastName, _customer.FirstName)
                End If
            End Get
            Protected Set(ByVal value As String)
                MyBase.DisplayName = value
            End Set
        End Property

        ''' <summary>
        ''' Gets/sets whether this customer is selected in the UI.
        ''' </summary>
        Public Property IsSelected() As Boolean
            Get
                Return _isSelected
            End Get
            Set(ByVal value As Boolean)
                If value = _isSelected Then
                    Return
                End If

                _isSelected = value

                MyBase.OnPropertyChanged("IsSelected")
            End Set
        End Property

        ''' <summary>
        ''' Returns a command that saves the customer.
        ''' </summary>
        Public ReadOnly Property SaveCommand() As ICommand
            Get
                If _saveCommand Is Nothing Then
                    Dim saveAction As New Action(Of Object)(AddressOf Me.Save)
                    _saveCommand = New RelayCommand(saveAction, Function(param) Me.CanSave)
                End If
                Return _saveCommand
            End Get
        End Property

#End Region ' Presentation Properties

#Region "Public Methods"

        ''' <summary>
        ''' Saves the customer to the repository.  This method is invoked by the SaveCommand.
        ''' </summary>
        Public Sub Save()
            If (Not _customer.IsValid) Then
                Throw New InvalidOperationException(Strings.CustomerViewModel_Exception_CannotSave)
            End If

            If Me.IsNewCustomer Then
                _customerRepository.AddCustomer(_customer)
            End If

            MyBase.OnPropertyChanged("DisplayName")
        End Sub

#End Region ' Public Methods

#Region "Private Helpers"

        ''' <summary>
        ''' Returns true if this customer was created by the user and it has not yet
        ''' been saved to the customer repository.
        ''' </summary>
        Private ReadOnly Property IsNewCustomer() As Boolean
            Get
                Return Not _customerRepository.ContainsCustomer(_customer)
            End Get
        End Property

        ''' <summary>
        ''' Returns true if the customer is valid and can be saved.
        ''' </summary>
        Private ReadOnly Property CanSave() As Boolean
            Get
                Return String.IsNullOrEmpty(Me.ValidateCustomerType()) AndAlso _customer.IsValid
            End Get
        End Property

#End Region ' Private Helpers

#Region "IDataErrorInfo Members"

        Private ReadOnly Property IDataErrorInfo_Error() As String Implements IDataErrorInfo.Error
            Get
                Return (TryCast(_customer, IDataErrorInfo)).Error
            End Get
        End Property

        Public ReadOnly Property IDataErrorInfo_Item(ByVal propertyName As String) As String Implements IDataErrorInfo.Item
            Get
                Dim [error] As String = Nothing

                If propertyName = "CustomerType" Then
                    ' The IsCompany property of the Customer class 
                    ' is Boolean, so it has no concept of being in
                    ' an "unselected" state.  The CustomerViewModel
                    ' class handles this mapping and validation.
                    [error] = Me.ValidateCustomerType()
                Else
                    [error] = (TryCast(_customer, IDataErrorInfo))(propertyName)
                End If

                ' Dirty the commands registered with CommandManager,
                ' such as our Save command, so that they are queried
                ' to see if they can execute now.
                CommandManager.InvalidateRequerySuggested()

                Return [error]
            End Get
        End Property

        Private Function ValidateCustomerType() As String
            If Me.CustomerType = Strings.CustomerViewModel_CustomerTypeOption_Company OrElse Me.CustomerType = Strings.CustomerViewModel_CustomerTypeOption_Person Then
                Return Nothing
            End If

            Return Strings.CustomerViewModel_Error_MissingCustomerType
        End Function

#End Region ' IDataErrorInfo Members
    End Class
End Namespace