Imports System.Linq
Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Windows.Data
Imports DemoApp.DataAccess
Imports DemoApp.Model
Imports DemoApp.Properties

Namespace DemoApp.ViewModel
    ''' <summary>
    ''' The ViewModel for the application's main window.
    ''' </summary>
    Public Class MainWindowViewModel
        Inherits WorkspaceViewModel
#Region "Fields"

        Private _commands As ReadOnlyCollection(Of CommandViewModel)
        Private ReadOnly _customerRepository As CustomerRepository
        Private _workspaces As ObservableCollection(Of WorkspaceViewModel)

#End Region ' Fields

#Region "Constructor"

        Public Sub New(ByVal customerDataFile As String)
            MyBase.DisplayName = Strings.MainWindowViewModel_DisplayName

            _customerRepository = New CustomerRepository(customerDataFile)
        End Sub

#End Region ' Constructor

#Region "Commands"

        ''' <summary>
        ''' Returns a read-only list of commands 
        ''' that the UI can display and execute.
        ''' </summary>
        Public ReadOnly Property Commands() As ReadOnlyCollection(Of CommandViewModel)
            Get
                If _commands Is Nothing Then
                    Dim cmds As List(Of CommandViewModel) = Me.CreateCommands()
                    _commands = New ReadOnlyCollection(Of CommandViewModel)(cmds)
                End If
                Return _commands
            End Get
        End Property

        Private Function CreateCommands() As List(Of CommandViewModel)
            Dim viewModel As New List(Of CommandViewModel)

            Dim showAllCustomersAction As New Action(Of Object)(AddressOf Me.ShowAllCustomers)
            Dim createNewCustomerAction As New Action(Of Object)(AddressOf Me.CreateNewCustomer)

            Dim viewAllCustomersView As CommandViewModel = New CommandViewModel(Strings.MainWindowViewModel_Command_ViewAllCustomers, New RelayCommand(showAllCustomersAction))
            Dim createNewCustomerView As New CommandViewModel(Strings.MainWindowViewModel_Command_CreateNewCustomer, New RelayCommand(createNewCustomerAction))

            viewModel.Add(viewAllCustomersView)
            viewModel.Add(createNewCustomerView)

            Return viewModel
        End Function

#End Region ' Commands

#Region "Workspaces"

        ''' <summary>
        ''' Returns the collection of available workspaces to display.
        ''' A 'workspace' is a ViewModel that can request to be closed.
        ''' </summary>
        Public ReadOnly Property Workspaces() As ObservableCollection(Of WorkspaceViewModel)
            Get
                If _workspaces Is Nothing Then
                    _workspaces = New ObservableCollection(Of WorkspaceViewModel)()
                    AddHandler _workspaces.CollectionChanged, AddressOf OnWorkspacesChanged
                End If
                Return _workspaces
            End Get
        End Property

        Private Sub OnWorkspacesChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
            If e.NewItems IsNot Nothing AndAlso e.NewItems.Count <> 0 Then
                For Each workspace As WorkspaceViewModel In e.NewItems
                    AddHandler workspace.RequestClose, AddressOf OnWorkspaceRequestClose
                Next workspace
            End If

            If e.OldItems IsNot Nothing AndAlso e.OldItems.Count <> 0 Then
                For Each workspace As WorkspaceViewModel In e.OldItems
                    RemoveHandler workspace.RequestClose, AddressOf OnWorkspaceRequestClose
                Next workspace
            End If
        End Sub

        Private Sub OnWorkspaceRequestClose(ByVal sender As Object, ByVal e As EventArgs)
            Dim workspace As WorkspaceViewModel = TryCast(sender, WorkspaceViewModel)
            workspace.Dispose()
            Me.Workspaces.Remove(workspace)
        End Sub

#End Region ' Workspaces

#Region "Private Helpers"

        Private Sub CreateNewCustomer()
            Dim newCustomer As Customer = Customer.CreateNewCustomer()
            Dim workspace As New CustomerViewModel(newCustomer, _customerRepository)
            Me.Workspaces.Add(workspace)
            Me.SetActiveWorkspace(workspace)
        End Sub

        Private Sub ShowAllCustomers()
            Dim workspace As AllCustomersViewModel = _
                CType(Me.Workspaces.FirstOrDefault(Function(vm) vm.GetType() Is GetType(AllCustomersViewModel)), AllCustomersViewModel)

            If workspace Is Nothing Then
                workspace = New AllCustomersViewModel(_customerRepository)
                Me.Workspaces.Add(workspace)
            End If

            Me.SetActiveWorkspace(workspace)
        End Sub

        Private Sub SetActiveWorkspace(ByVal workspace As WorkspaceViewModel)
            Debug.Assert(Me.Workspaces.Contains(workspace))

            Dim collectionView As ICollectionView = CollectionViewSource.GetDefaultView(Me.Workspaces)
            If collectionView IsNot Nothing Then
                collectionView.MoveCurrentTo(workspace)
            End If
        End Sub

#End Region ' Private Helpers
    End Class
End Namespace